import elf;
import elfsymtab;
import std.stdio;
import std.conv;

void addr2sym(ElfSymTab symtab, uint addr)
{
    try
    {
	Elf32_Sym sym = symtab.findSymbol(addr);
	writefln("0x%x: in [0x%x, %d] %s %s + %d (%s)", addr, sym.st_value, sym.st_size, 
		 DEF_SYM_TYPES()[ELF32_ST_TYPE(sym.st_info)], symtab.name(sym), addr-sym.st_value,
		 symtab.name(symtab.section(sym)));
    }
    catch(Error e)
    {
	try
	{
	    Elf32_Shdr sec = symtab.findSection(addr);
	    writefln("0x%x: in [0x%x, %d] section %s", addr, sec.sh_addr, sec.sh_size, symtab.name(sec));
	}
	catch(Error e)
	{
	    writefln("0x%x not found", addr);
	}
    }
}

int main(string[] args)
{
    if(args.length < 2) {
        writefln("usage: %s <ELF file> [hexaddr1 ... hexaddrN]", args[0]);
	return 1;
    }
    auto symtab = new ElfSymTab();
    string filename = args[1];
    symtab.parseFile(filename);
    foreach(string s; args[2 .. args.length])
    {
	uint addr;
	if(!sscanf(cast(char*)(s),cast(char*)("%x"),&addr))
	{
	    writefln("%s: expected a hexadecimal address", s);
	}
	else {
            addr2sym(symtab, addr);
	}
    }
    return 0;
}
